# -*- encoding: utf-8 -*-
##############################################################################
#
#    Addition to OpenERP, Open Source Management Solution   
#    Copyright (C) 2012 Vertel. All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields,osv
from mail.mail_message import to_email
import re
from tools.translate import _
import pdb

class sale_order(osv.osv):
    _name = 'sale.order'
    _inherit = ['sale.order','mail.thread']

    _defaults = {
        'partner_id': 1,
        'pricelist_id': 1,
        'partner_invoice_id': lambda self, cr, uid, context: context.get('partner_id', False) and self.pool.get('res.partner').address_get(cr, uid, [context['partner_id']], ['invoice'])['invoice'] or 1,
        'partner_order_id': lambda self, cr, uid, context: context.get('partner_id', False) and  self.pool.get('res.partner').address_get(cr, uid, [context['partner_id']], ['contact'])['contact'] or 1,
        'partner_shipping_id': lambda self, cr, uid, context: context.get('partner_id', False) and self.pool.get('res.partner').address_get(cr, uid, [context['partner_id']], ['delivery'])['delivery'] or 1,
    
    }
    
    def message_new(self, cr, uid, msg, custom_values=None, context=None):
        res_id = super(sale_order,self).message_new(cr, uid, msg, custom_values=custom_values, context=context)
        subject = msg.get('subject')
        body = msg.get('body_text')
        msg_from = msg.get('from')
        data = {
            'origin': "%s %s" % (msg_from,subject),
            'note': body,
        }
        partner_id = self.message_partner_by_email(cr, uid, msg_from)['partner_id']
        if not partner_id:
            partner_id = 1
        data.update({'partner_id': partner_id })
        self.log(cr, uid, res_id, _('Sale order created %s subject %s from  %s') % (res_id,subject,msg_from))

        self.write(cr, uid, [res_id], data, context)

        pdb.set_trace()
        
        for line in body.splitlines():
            try:
                product,qty = re.match('\s*([a-zA-z0-9-#]+)\s+(\d+)\s*',line).groups()
            except:
                try:
                    product = re.match('\s*([a-zA-z0-9-#]+)\s*',line).groups()
                    qty = 1
                except:
                    product = None
                    qty = 0
            if product:
                if len(product)>0:
                    product = product[0]
                self.log(cr, uid, res_id, _('Found product %s and qty %s in line %s') % (product,qty,line))
#                product_id = self.pool.get('product.product').search(cr,uid,['|','|',('name','ilike',product),('default_code','=',product),('ean13','=',product)],0,context=context)
                product_id = self.pool.get('product.product').search(cr,uid,['|',('default_code','=',product),('ean13','=',product)],0,context=context)
                if len(product_id) > 0:
                    self.log(cr, uid, res_id, _('Found product %s in database %s') % (product,product_id))
                    partner = self.pool.get('res.partner').browse(cr,uid,data['partner_id'])                    
                    line_id = self.pool.get('sale.order.line').create(cr,uid, {'order_id': res_id, 'product_id': 0,'name': '*nothing yet*',})                    
                    res = self.pool.get('sale.order.line').product_id_change(cr, uid, [line_id], partner.property_product_pricelist.id, product_id[0], qty_uos=qty, partner_id=data['partner_id'])
                    self.log(cr, uid, res_id, _('Product id change value %s warning %s domain %s ') % (res['value'],res['warning'],res['domain']))
                    self.pool.get('sale.order.line').write(cr,uid, line_id, res['value'])                    
        return res_id

    def message_update(self, cr, uid, ids, msg, data={}, default_act='pending'):
        data.update({
            'note': msg['body_text'],
        })
        act = 'do_'+default_act

        self.write(cr, uid, ids, data, context=context)
        getattr(self,act)(cr, uid, ids, context=context)
        self.message_append_dict(cr, uid, [res_id], msg, context=context)
        return True

    def message_thread_followers(self, cr, uid, ids, context=None):
        followers = super(sale_order,self).message_thread_followers(cr, uid, ids, context=context)
        for task in self.browse(cr, uid, followers.keys(), context=context):
            task_followers = set(followers[task.id])
            task_followers.add(task.user_id.user_email)
            followers[task.id] = filter(None, task_followers)
        return followers

    def do_draft(self, cr, uid, ids, context=None):
        res = super(sale_order, self).do_draft(cr, uid, ids, context)
        tasks = self.browse(cr, uid, ids, context=context)
        self.message_append(cr, uid, tasks, _('Draft'), context=context)
        return res

    def do_open(self, cr, uid, ids, context=None):
        res = super(sale_order, self).do_open(cr, uid, ids, context)
        tasks = self.browse(cr, uid, ids, context=context)
        self.message_append(cr, uid, tasks, _('Open'), context=context)
        return res

    def do_pending(self, cr, uid, ids, context=None):
        res = super(sale_order, self).do_pending(cr, uid, ids, context)
        tasks = self.browse(cr, uid, ids, context=context)
        self.message_append(cr, uid, tasks, _('Pending'), context=context)
        return res

    def do_close(self, cr, uid, ids, context=None):
        res = super(sale_order, self).do_close(cr, uid, ids, context)
        tasks = self.browse(cr, uid, ids, context=context)
        for task in tasks:
            if task.state == 'done':
                self.message_append(cr, uid, tasks, _('Done'), context=context)
        return res

    def do_cancel(self, cr, uid, ids, context=None):
        res = super(sale_order, self).do_cancel(cr, uid, ids, context=context)
        tasks = self.browse(cr, uid, ids, context=context)
        self.message_append(cr, uid, tasks, _('Cancel'), context=context)
        return res

sale_order()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
